@@ -151,6 +151,26 @@ bool DualShock4Mapper(uint16_t type,
   return DO_MAPPING;
 }
 
+bool NintendoSwitchPro(uint16_t type,
+                       uint16_t code,
+                       GamepadEventType* mapped_type,
+                       uint16_t* mapped_code) {
+  static const KeyMapType key_mapping = {
+      {BTN_A, WG_BUTTON_A},           {BTN_B, WG_BUTTON_B},
+      {BTN_C, WG_BUTTON_X},           {BTN_X, WG_BUTTON_Y},
+      {BTN_Y, WG_BUTTON_L1},          {BTN_Z, WG_BUTTON_R1},
+      {BTN_TL, WG_BUTTON_LT},         {BTN_TR, WG_BUTTON_RT},
+      {BTN_TL2, WG_BUTTON_SELECT},    {BTN_TR2, WG_BUTTON_START},
+      {BTN_SELECT, WG_BUTTON_THUMBL}, {BTN_START, WG_BUTTON_THUMBR},
+      {BTN_MODE, WG_BUTTON_MODE}};
+
+  static const AbsMapType abs_mapping = {
+      TO_ABS(ABS_X, WG_ABS_X),   TO_ABS(ABS_Y, WG_ABS_Y),
+      TO_ABS(ABS_RX, WG_ABS_RX), TO_ABS(ABS_RY, WG_ABS_RY),
+      TO_BTN(ABS_HAT0X, kHAT_X), TO_BTN(ABS_HAT0Y, kHAT_Y)};
+  return DO_MAPPING;
+}
+
 bool NintendoSwitchLeft(uint16_t type,
                         uint16_t code,
                         GamepadEventType* mapped_type,
@@ -459,12 +479,15 @@ static const struct MappingData {
     {0x054c, 0x0268, PlaystationSixAxisMapper},  // Playstation 3.
     {0x054c, 0x05c4, DualShock4Mapper},          // Dualshock 4.
     // Nintendo switch.
+    {0x057e, 0x2009, NintendoSwitchPro},    // Nintendo switch pro.
     {0x057e, 0x2006, NintendoSwitchLeft},   // Nintendo switch left.
     {0x057e, 0x2007, NintendoSwitchRight},  // Nintendo switch right.
     // NES style gamepad.
-    {0x0583, 0x2060, IBuffalocClassicMapper},  // iBuffalo Classic.
-    {0x0079, 0x0011, ClassicNESMapper},        // Classic NES controller.
-    {0x12bd, 0xd015, SNesRetroMapper},         // Hitgaming SNES retro.
+    {0x0583, 0x2060, IBuffalocClassicMapper},         // iBuffalo Classic.
+    {0x0079, 0x0011, ClassicNESMapper},               // Classic NES controller.
+    {0x12bd, 0xd015, SNesRetroMapper},                // Hitgaming SNES retro.
+    {0x3810, 0x9, Vendor_046dProduct_b501Mapper},     // FC30 Pro, bluetooth.
+    {0x1002, 0x9000, Vendor_046dProduct_b501Mapper},  // FC30 Pro, wired.
     // Android gamepad.
     {0x0b05, 0x4500, ADT1Mapper},  // Nexus player controller (asus gamepad).
     {0x1532, 0x0900, ADT1Mapper},  // Razer serval.
