@@ -10,6 +10,7 @@
 #include <vsstyle.h>
 #include <vssym32.h>
 
+#include "base/command_line.h"
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/win/scoped_gdi_object.h"
@@ -28,6 +29,7 @@
 #include "third_party/skia/include/core/SkShader.h"
 #include "third_party/skia/include/core/SkSurface.h"
 #include "ui/base/material_design/material_design_controller.h"
+#include "ui/base/ui_base_switches.h"
 #include "ui/display/win/screen_win.h"
 #include "ui/gfx/color_palette.h"
 #include "ui/gfx/color_utils.h"
@@ -149,11 +151,6 @@ NativeTheme* NativeTheme::GetInstanceForNativeUi() {
   return NativeThemeWin::instance();
 }
 
-// static
-bool NativeThemeWin::IsUsingHighContrastTheme() {
-  return instance()->IsUsingHighContrastThemeInternal();
-}
-
 // static
 void NativeThemeWin::CloseHandles() {
   instance()->CloseHandlesInternal();
@@ -265,14 +262,14 @@ NativeThemeWin::NativeThemeWin()
 
 NativeThemeWin::~NativeThemeWin() {
   if (theme_dll_) {
-    // todo (cpu): fix this soon.  Making a call to CloseHandles() here breaks
+    // TODO(https://crbug.com/787692): Calling CloseHandles() here breaks
     // certain tests and the reliability bots.
     // CloseHandles();
     FreeLibrary(theme_dll_);
   }
 }
 
-bool NativeThemeWin::IsUsingHighContrastThemeInternal() {
+bool NativeThemeWin::IsUsingHighContrastThemeInternal() const {
   if (is_using_high_contrast_valid_)
     return is_using_high_contrast_;
   HIGHCONTRAST result;
@@ -457,8 +454,12 @@ SkColor NativeThemeWin::GetSystemColor(ColorId color_id) const {
 
     // FocusableBorder
     case kColorId_FocusedBorderColor:
+      if (ui::MaterialDesignController::IsSecondaryUiMaterial())
+        break;
       return kFocusedBorderColor;
     case kColorId_UnfocusedBorderColor:
+      if (ui::MaterialDesignController::IsSecondaryUiMaterial())
+        break;
       return kUnfocusedBorderColor;
 
     // Button
@@ -510,8 +511,8 @@ SkColor NativeThemeWin::GetSystemColor(ColorId color_id) const {
     case kColorId_TreeSelectionBackgroundFocused:
       return system_colors_[COLOR_HIGHLIGHT];
     case kColorId_TreeSelectionBackgroundUnfocused:
-      return system_colors_[IsUsingHighContrastTheme() ?
-                              COLOR_MENUHIGHLIGHT : COLOR_BTNFACE];
+      return system_colors_[UsesHighContrastColors() ? COLOR_MENUHIGHLIGHT
+                                                     : COLOR_BTNFACE];
 
     // Table
     case kColorId_TableBackground:
@@ -525,8 +526,8 @@ SkColor NativeThemeWin::GetSystemColor(ColorId color_id) const {
     case kColorId_TableSelectionBackgroundFocused:
       return system_colors_[COLOR_HIGHLIGHT];
     case kColorId_TableSelectionBackgroundUnfocused:
-      return system_colors_[IsUsingHighContrastTheme() ?
-                              COLOR_MENUHIGHLIGHT : COLOR_BTNFACE];
+      return system_colors_[UsesHighContrastColors() ? COLOR_MENUHIGHLIGHT
+                                                     : COLOR_BTNFACE];
     case kColorId_TableGroupingIndicatorColor:
       return system_colors_[COLOR_GRAYTEXT];
 
@@ -621,6 +622,12 @@ gfx::Rect NativeThemeWin::GetNinePatchAperture(Part part) const {
   return gfx::Rect();
 }
 
+bool NativeThemeWin::UsesHighContrastColors() const {
+  bool force_enabled = base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kForceHighContrast);
+  return force_enabled || IsUsingHighContrastThemeInternal();
+}
+
 void NativeThemeWin::PaintIndirect(cc::PaintCanvas* destination_canvas,
                                    Part part,
                                    State state,
@@ -715,7 +722,9 @@ void NativeThemeWin::PaintIndirect(cc::PaintCanvas* destination_canvas,
     }
   }
 
-  destination_canvas->drawBitmap(offscreen_bitmap, rect.x(), rect.y());
+  destination_canvas->drawImage(
+      cc::PaintImage::CreateFromBitmap(std::move(offscreen_bitmap)), rect.x(),
+      rect.y());
 }
 
 HRESULT NativeThemeWin::GetThemePartSize(ThemeName theme_name,
