@@ -8,12 +8,12 @@
 #include <stdint.h>
 
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkCanvas.h"
 #include "third_party/skia/include/core/SkSurface.h"
+#include "ui/gfx/gpu_fence.h"
 #include "ui/gfx/presentation_feedback.h"
 #include "ui/ozone/common/gpu/ozone_gpu_message_params.h"
 #include "ui/ozone/platform/drm/common/drm_util.h"
@@ -115,12 +115,15 @@ void DrmWindow::MoveCursor(const gfx::Point& location) {
     controller_->MoveCursor(location);
 }
 
-bool DrmWindow::SchedulePageFlip(const std::vector<OverlayPlane>& planes,
-                                 SwapCompletionOnceCallback callback) {
+void DrmWindow::SchedulePageFlip(
+    std::vector<DrmOverlayPlane> planes,
+    SwapCompletionOnceCallback submission_callback,
+    PresentationOnceCallback presentation_callback) {
   if (controller_) {
-    const DrmDevice* drm = controller_->GetAllocationDrmDevice().get();
+    const DrmDevice* drm = controller_->GetDrmDevice().get();
     for (const auto& plane : planes) {
-      if (plane.buffer && plane.buffer->GetDrmDevice() != drm) {
+      if (plane.buffer &&
+          plane.buffer->GetGbmDeviceLinux() != drm->AsGbmDeviceLinux()) {
         // Although |force_buffer_reallocation_| is set to true during window
         // bounds update, this may still be needed because of in-flight buffers.
         force_buffer_reallocation_ = true;
@@ -131,21 +134,23 @@ bool DrmWindow::SchedulePageFlip(const std::vector<OverlayPlane>& planes,
 
   if (force_buffer_reallocation_) {
     force_buffer_reallocation_ = false;
-    std::move(callback).Run(gfx::SwapResult::SWAP_NAK_RECREATE_BUFFERS,
-                            gfx::PresentationFeedback());
-    return true;
+    std::move(submission_callback)
+        .Run(gfx::SwapResult::SWAP_NAK_RECREATE_BUFFERS, nullptr);
+    std::move(presentation_callback).Run(gfx::PresentationFeedback::Failure());
+    return;
   }
 
-  last_submitted_planes_ = planes;
+  last_submitted_planes_ = DrmOverlayPlane::Clone(planes);
 
   if (!controller_) {
-    std::move(callback).Run(gfx::SwapResult::SWAP_ACK,
-                            gfx::PresentationFeedback());
-    return true;
+    std::move(submission_callback).Run(gfx::SwapResult::SWAP_ACK, nullptr);
+    std::move(presentation_callback).Run(gfx::PresentationFeedback::Failure());
+    return;
   }
 
-  return controller_->SchedulePageFlip(last_submitted_planes_,
-                                       std::move(callback));
+  controller_->SchedulePageFlip(std::move(planes),
+                                std::move(submission_callback),
+                                std::move(presentation_callback));
 }
 
 std::vector<OverlayCheckReturn_Params> DrmWindow::TestPageFlip(
@@ -154,8 +159,8 @@ std::vector<OverlayCheckReturn_Params> DrmWindow::TestPageFlip(
                                           last_submitted_planes_);
 }
 
-const OverlayPlane* DrmWindow::GetLastModesetBuffer() {
-  return OverlayPlane::GetPrimaryPlane(last_submitted_planes_);
+const DrmOverlayPlane* DrmWindow::GetLastModesetBuffer() {
+  return DrmOverlayPlane::GetPrimaryPlane(last_submitted_planes_);
 }
 
 void DrmWindow::GetVSyncParameters(
@@ -212,7 +217,7 @@ void DrmWindow::SetController(HardwareDisplayController* controller) {
 
   controller_ = controller;
   device_manager_->UpdateDrmDevice(
-      widget_, controller ? controller->GetAllocationDrmDevice() : nullptr);
+      widget_, controller ? controller->GetDrmDevice() : nullptr);
 
   UpdateCursorBuffers();
   // We changed displays, so we want to update the cursor as well.
@@ -226,7 +231,7 @@ void DrmWindow::UpdateCursorBuffers() {
       cursor_buffers_[i] = nullptr;
     }
   } else {
-    scoped_refptr<DrmDevice> drm = controller_->GetAllocationDrmDevice();
+    scoped_refptr<DrmDevice> drm = controller_->GetDrmDevice();
     gfx::Size max_cursor_size = GetMaximumCursorSize(drm->get_fd());
     SkImageInfo info = SkImageInfo::MakeN32Premul(max_cursor_size.width(),
                                                   max_cursor_size.height());
