@@ -53,6 +53,8 @@
 #include "renderer_host/user_resource_controller_host.h"
 #include "web_engine_settings.h"
 
+#include <QtWebEngineCore/qwebengineurlscheme.h>
+
 using QtWebEngineCore::ProfileAdapter;
 
 QT_BEGIN_NAMESPACE
@@ -857,7 +859,7 @@ static bool checkInternalScheme(const QByteArray &scheme)
 /*!
     Registers a handler \a handler for custom URL scheme \a scheme in the profile.
 
-    It is recommended to first register the scheme with \l
+    It is necessary to first register the scheme with \l
     QWebEngineUrlScheme::registerScheme at application startup.
 */
 void QQuickWebEngineProfile::installUrlSchemeHandler(const QByteArray &scheme, QWebEngineUrlSchemeHandler *handler)
@@ -875,6 +877,11 @@ void QQuickWebEngineProfile::installUrlSchemeHandler(const QByteArray &scheme, Q
             qWarning("URL scheme handler already installed for the scheme: %s", scheme.constData());
         return;
     }
+
+    if (QWebEngineUrlScheme::schemeByName(canonicalScheme) == QWebEngineUrlScheme())
+        qWarning("Please register the custom scheme '%s' via QWebEngineUrlScheme::registerScheme() "
+                 "before installing the custom scheme handler.", scheme.constData());
+
     d->profileAdapter()->addCustomUrlSchemeHandler(canonicalScheme, handler);
     connect(handler, SIGNAL(_q_destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)), this, SLOT(destroyedUrlSchemeHandler(QWebEngineUrlSchemeHandler*)));
 }
